require_relative 'player'
require_relative 'die'
require_relative 'game_turn'
require_relative 'treasure_trove'
require 'csv'
module StudioGame
	class Game
		attr_reader :title
		
		def initialize(title)
			@title = title
			@players = []
		end

		def load_players(input_file)
			CSV.foreach(input_file) do |row|
				player = Player.new(row[0], row[1].to_i)
				add_player(player)
			end
		end

		def save_high_scores(to_file = "high_scores.txt")
			File.open(to_file, "w") do |file|
				file.puts "#{@title} High Scores:"
				@players.sort.each do |player|
					file.puts high_score_entry(player)
				end
			end
		end

		def add_player(player)
			@players << player
		end

		def print_name_and_health(player)
			puts "#{player.name} (#{player.health})"
		end

		def total_points
			@players.reduce(0) {|sum, player| sum + player.points}
		end

		def high_score_entry(player)
			formatted_name = player.name.ljust(20, '.')
			"#{formatted_name} #{player.score}"				
		end

		def print_stats
			strong, wimpy = @players.partition {|p| p.strong?}
			puts "\n#{@title} Statistics:\n"
			puts "#{strong.size} strong players:"
			strong.each do |p|
				print_name_and_health(p)
			end
			puts "#{wimpy.size} wimpy players:"
			wimpy.each do |p|
				print_name_and_health(p)
			end
			sorted_players = @players.sort {|a, b| b.score <=> a.score}
			puts "\n#{@title} High Scores:"
			sorted_players.each do |p|
				puts high_score_entry(p)
			end

			@players.each do |player|
				puts "\n#{player.name}'s point totals:"
				player.each_found_treasure do |treasure|
					puts "#{treasure.points} total #{treasure.name} points"
				end
				puts "#{player.points} grand total points"
			end

			puts "\n#{total_points} total points from treasures found"
		end

		def play(rounds)
			puts "There are #{@players.size} players in #{@title}"
			@players.each do |player|
				puts player
			end

			treasures = TreasureTrove::TREASURES
			puts "\nThere are #{treasures.size} treasures to be found:"
			treasures.each do |treasure|
				puts "A #{treasure.name} is work #{treasure.points} points"
			end

			1.upto(rounds) do | round |
				if block_given?
					break if yield 
				end
				puts "\nRound number #{round}:"
				@players.each do |player|
					GameTurn.take_turn(player)
					puts player
				end
			end
		end
	end
end
if __FILE__ == $0
	player1 = Player.new("moe", 10)
	player2 = Player.new("curly")
	game1 = Game.new("Knuckheads")
	game1.add_player(player1)
	game1.add_player(player2)
	game1.play(2)
end
